"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addEpisodeNamesToInput = addEpisodeNamesToInput;
exports.addNumberOfSeasonsToInput = addNumberOfSeasonsToInput;
exports.getEpisodePathsWithEpisodeNames = getEpisodePathsWithEpisodeNames;
exports.renameEpisodes = renameEpisodes;
exports.renameSeasons = renameSeasons;

require("core-js/modules/es.promise.js");

require("core-js/modules/es.array.reduce.js");

require("core-js/modules/web.dom-collections.iterator.js");

require("core-js/modules/es.array.sort.js");

var _shelljs = require("shelljs");

var _inquirer = _interopRequireDefault(require("inquirer"));

var _finders = require("./finders");

var _imdb = require("./imdb");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Finds the number of seasons listed on IMDb for all provided seasons
 *
 * @param {object} input Blob of input data passed through program
 * @returns {object} Modified blob of input data
 */
async function addNumberOfSeasonsToInput(input) {
  return Object.keys(input).reduce(async (acc, item) => {
    const seasonLink = await (0, _imdb.getSeriesLink)(0, input[item].imdbId);
    const numSeasons = await (0, _imdb.getNumSeries)(seasonLink);
    return acc.then(newAcc => _objectSpread(_objectSpread({}, newAcc), {}, {
      [item]: _objectSpread(_objectSpread({}, input[item]), {}, {
        numSeasons
      })
    }));
  }, Promise.resolve({}));
}
/**
 * Finds all of the episodes for a given IMDb ID
 *
 * @param {number} numSeasons The number of seasons
 * @param {string} imdbId The IMDb ID of the show
 * @returns {object} Modified blob of input data
 */


function fetchEpisodeNamesForShow(numSeasons, imdbId) {
  return [...Array(numSeasons).keys()].reduce(async (episodeNumsAcc, seasonNum) => {
    const seasonLink = await (0, _imdb.getSeriesLink)(seasonNum + 1, imdbId);
    const episodeNames = await (0, _imdb.getEpisodeNames)(seasonLink);
    return episodeNumsAcc.then(newEpisodeNumsAcc => _objectSpread(_objectSpread({}, newEpisodeNumsAcc), {}, {
      [seasonNum + 1]: episodeNames
    }));
  }, Promise.resolve({}));
}
/**
 * Finds all of the episodes for all of the input shows
 *
 * @param {object} input Blob of input data passed through program
 * @returns {object} Modified blob of input data
 */


async function addEpisodeNamesToInput(input) {
  return Object.keys(input).reduce(async (acc, key) => {
    const episodeNames = await fetchEpisodeNamesForShow(input[key].numSeasons, input[key].imdbId);
    return acc.then(newAcc => _objectSpread(_objectSpread({}, newAcc), {}, {
      [key]: _objectSpread(_objectSpread({}, input[key]), {}, {
        episodeNames
      })
    }));
  }, Promise.resolve({}));
}
/**
 * Pairs all of the episode paths with the name of that episode
 *
 * @param {object} input Blob of input data passed through program
 * @returns {Array<Array<object>>} An array of seasons which are an array of objects
 * each with the path to the episode, and the name of that episode
 */


function getEpisodePathsWithEpisodeNames(input) {
  return Object.values(input).map(blob => {
    const seasonPaths = (0, _finders.findDirectoriesAtPath)(blob.tvShowRootPath).sort();
    const seasonEpisodePaths = seasonPaths.map(_finders.findVideoPaths).map(array => array.sort());
    return seasonEpisodePaths.map((epsForSeason, seasonIndex) => epsForSeason.map((episodePath, episodeIndex) => ({
      path: episodePath,
      name: blob.episodeNames["".concat(seasonIndex + 1)][episodeIndex]
    })));
  });
}
/**
 * Renames every file found at the input paths, to an episode with a number,
 * and the name of the episode
 *
 * @param {Array<Array<object>>} input An array of seasons which are an array of objects
 * each with the path to the episode, and the name of that episode
 */


async function renameEpisodes(input) {
  for (let i = 0; i < input.length; i += 1) {
    for (let j = 0; j < input[i].length; j += 1) {
      const episodeItem = input[i][j];
      const episodeRoot = (0, _finders.getRootPathItem)(episodeItem.path);
      const extension = (0, _finders.getPathExtension)(episodeItem.path);
      const newNamePath = "".concat(episodeRoot, "/Episode ").concat(String(j + 1).padStart(2, '0'), " - ").concat(episodeItem.name, ".").concat(extension);

      if (episodeItem.path === newNamePath) {
        console.log('Old path would the same as the new path, skipping - ', episodeItem.path);
      } else {
        console.log('This is the suggested renaming:');
        console.log(episodeItem.path);
        console.log(newNamePath);
        await _inquirer.default.prompt([{
          name: 'confirmation',
          type: 'confirm',
          message: 'Sound good?'
        }]).then(answer => {
          if (answer.confirmation) {
            (0, _shelljs.mv)(episodeItem.path, newNamePath);
          }
        });
      }
    }
  }
}
/**
 * Renames every season folder to Season X
 *
 * @param {object} input Blob of input data passed through program
 */


async function renameSeasons(input) {
  const seasonPathsArray = Object.values(input).map(path => (0, _finders.findDirectoriesAtPath)(path.tvShowRootPath));

  for (let i = 0; i < seasonPathsArray.length; i += 1) {
    for (let j = 0; j < seasonPathsArray[i].length; j += 1) {
      const seasonPath = seasonPathsArray[i][j];
      const root = (0, _finders.getRootPathItem)(seasonPath);
      const newPath = "".concat(root, "/Season ").concat(String(j + 1).padStart(2, '0'));

      if (seasonPath === newPath) {
        console.log('Old path would the same as the new path, skipping - ', seasonPath);
      } else {
        console.log('This is the suggested renaming:');
        console.log(seasonPath);
        console.log(newPath);
        await _inquirer.default.prompt([{
          name: 'confirmation',
          type: 'confirm',
          message: 'Sound good?'
        }]).then(answer => {
          if (answer.confirmation) {
            (0, _shelljs.mv)(seasonPath, newPath);
          }
        });
      }
    }
  }
}